(************************************************************ 
 Amalthea main                                                
   Amalthea, amalthea.ml                                      
   Martin Sandin (d97masa@dtek.chalmers.se)                   
 ************************************************************)
open Io_types

(* The version of Amalthea *)
let amalthea_version = 0.875


(* Flags *)
let filename = ref ""
let pretty_flag = ref false
let version_flag = ref false

(* Argument handling *)
let usage_msg =
  "Usage: amalthea [options] <sourcefile>\n\
   Runs the Io source file.\n\
   Options are:"
let speclist =
  [("-noprelude", Arg.Unit (fun _ -> Io_module.default_imports := []), " prevent the prelude from being loaded");    
   ("-prelude <name>", Arg.String (fun n -> Io_module.default_imports := [(n,None)]), " load a different module as prelude");
   ("-pretty", Arg.Set pretty_flag, " pretty print the source file");
   ("-trace", Arg.Set Io_interpreter.trace_flag, " print execution trace");
   ("-vars", Arg.Set Io_env.variable_flag, " enable mutable module variables (unpure)");   
   ("-version", Arg.Set version_flag, " print the Amalthea version number")]


(* Loads dependant modules and interprets the program *)
let build_n_run : string * io_ast -> unit
= function (_, (_, _, _, ex)) as p ->
  let world = Io_module.catch_module_errors
      Io_module.load_world_of_ast p in
  Io_interpreter.catch_interpreter_errors
    Io_interpreter.interpret_expr_in_world (world, ex)


(* And the program starts here *)
let _ =
  Arg.parse speclist (fun f -> filename := f) usage_msg;
  if !version_flag then
    print_string ("Amalthea version " ^ string_of_float amalthea_version
		   ^ "")
  else 
    if !filename = "" then
      (print_endline "Amalthea: missing argument sourcefile";
      Arg.usage speclist usage_msg)
    else
      try
        let ast = Io_module.catch_module_errors Io_module.load_ast !filename  in
        if !pretty_flag then
          print_string (Io_pretty.pretty_print ast)
        else
          build_n_run (!filename, ast);
          flush stdout
      with
        | Sys_error error -> print_endline ("Amalthea: " ^ error)
        | Io_error error -> print_endline error
