(* Misc utility functions relating to files *)

let rec find_file : string -> string -> string
= fun dir_name file_name ->
  if Sys.file_exists (Filename.concat dir_name file_name) then
    Filename.concat dir_name file_name
  else
    let dir = Unix.opendir dir_name in
    let path = ref "" in
      (try
        while !path = "" do
          let entry = Unix.readdir dir in
          match entry with
            | "." -> ()
            | ".." -> ()
            | _ ->
              let entry = Filename.concat dir_name entry in
              match (Unix.stat entry).Unix.st_kind with
                | Unix.S_DIR ->
                  (try
                    path := find_file entry file_name
                  with
                    | Unix.Unix_error (Unix.ENOENT, _, _) -> ())
                | _ -> ()
        done
      with
        End_of_file -> ());
      if (!path) = "" then
        raise (Unix.Unix_error (Unix.ENOENT, "find_file", file_name))
      else
        !path
