{
open Io_parser;;
open Io_types;;

let row = ref 1
let row_start = ref 1
let module_name = ref ""
let current_pos = ref (!module_name,!row,1)
let reset_pos = function mname ->
  module_name := mname;
  row := 1;
  row_start := 1
  
let pos = function lexbuf ->
  current_pos := (!module_name, !row, (Lexing.lexeme_start lexbuf) - (!row_start));
  !current_pos
  
let string_chars s = String.sub s 1 ((String.length s)-2) ;;
  
}
let idch = ['A'-'z' '=' '*' '/' '-' '+' '>' '<' '^' '_' '%' '@' '!' '?' '&' '|' '$' '\\' '~']
rule lexer = parse
  [' ' '\t' '\r']              { lexer lexbuf }
  | '\n'                       { row := (!row) + 1;row_start := Lexing.lexeme_start lexbuf;lexer lexbuf }
  | '#' [^ '\n']* '\n'         { row := (!row) + 1;row_start := Lexing.lexeme_start lexbuf;lexer lexbuf }
  | eof                        { Lend }
  | "("                        { Llpar }
  | ")"                        { Lrpar }
  | ";"                        { Lsemi }
  | ":"                        { Lcolon }
  | "."                        { Lperiod }
  | "->"                       { Llambda  (pos lexbuf) }
  | "<<="                      { Lputvar  (pos lexbuf) }
  | "=>>"                      { Lgetvar  (pos lexbuf) }
  | "declare"                  { Ldeclare (pos lexbuf) }
  | "variable"                 { Lvariable (pos lexbuf) }
  | "primitive"                { Lprimitive (pos lexbuf) }
  | "import"                   { Limport (pos lexbuf) }
  | "export"                   { Lexport (pos lexbuf) }
  | '-'? ['0'-'9']+            { Lint  (pos lexbuf,(int_of_string (Lexing.lexeme lexbuf))) }
  | '"' [^ '"']* '"'           { Lstring (pos lexbuf,string_chars (Lexing.lexeme lexbuf)) }
  | idch ( idch | ['0'-'9'] )* { Lident (pos lexbuf,(Lexing.lexeme lexbuf)) }

{

let lex_source =
function i ->
  try
    lexer i
  with
    Failure e ->
      raise (Io_module_error (Io_lexer_error !current_pos))
}