(************************************************************ 
 Io Default Primitive Continuation Functions                  
   Amalthea, ml                                 
   Martin Sandin (d97masa@dtek.chalmers.se)                   
 ************************************************************)
 
open Io_types;;

(* Misc operations *)
let terminate = function
  | [] -> (Cterminate, [])
  | _ -> raise (Io_prim_error "terminate")

(* Integer operations *)
let addint = function
  | [Cint na; Cint nb; c] -> (c, [Cint (na + nb)])
  | _ -> raise (Io_prim_error "+")

let multint = function
  | [Cint na; Cint nb; c] -> (c, [Cint (na * nb)])
  | _ -> raise (Io_prim_error "*") 

let divint = function
  | [Cint na; Cint nb; c] ->
    (try
      (c, [Cint (na / nb)])
    with
      | Division_by_zero -> raise (Io_prim_error "/: division by zero"))
  | _ -> raise (Io_prim_error "/") 
  
let subint = function
  | [Cint na; Cint nb; c] -> (c, [Cint (na - nb)])
  | _ -> raise (Io_prim_error "-") 
  
let eqcont = function
  | [Cint na;Cint nb; ctrue; cfalse] -> ((if na = nb then ctrue else cfalse), [])
  | [Cstring na; Cstring nb; ctrue; cfalse] -> ((if na = nb then ctrue else cfalse), [])
  | [ca; cb; ctrue; cfalse] -> (cfalse,[])
  | _ -> raise (Io_prim_error "=")

let ltint = function
  | [Cint na; Cint nb; ctrue; cfalse] -> ((if na < nb then ctrue else cfalse), [])
  | _ -> raise (Io_prim_error "<")

let gtint = function
  | [Cint na; Cint nb; ctrue; cfalse] -> ((if na > nb then ctrue else cfalse), [])
  | _ -> raise (Io_prim_error ">")

let printint = function
  | [Cint n; c] -> print_int n;print_newline (); (c, [])
  | _ -> raise (Io_prim_error "print_int")

let printint_ = function
  | [Cint n; c] -> print_int n; (c, [])
  | _ -> raise (Io_prim_error "print_int_")

let stringfromint = function
  | [Cint n; c] -> (c, [Cstring (string_of_int n)])
  | _ -> raise (Io_prim_error "string_of_int")

let intfromstring = function
  | [Cstring n; c] ->
    (try
      (c, [Cint (int_of_string n)])
    with
      Failure _ -> raise (Io_prim_error "int_of_string: string not valid integer"))
  | _ -> raise (Io_prim_error "int_of_string")

(* String operation *)
let concstring = function
  | [Cstring sa; Cstring sb; c] -> (c, [Cstring (sa ^ sb)])
  | _ -> raise (Io_prim_error "^")

let substring = function
  | [Cstring sa; Cint is; Cint il; c] -> (c, [Cstring (String.sub sa is il)])
  | _ -> raise (Io_prim_error "substring")
  
let string_length = function
  | [Cstring sa; c] -> (c, [Cint (String.length sa)])
  | _ -> raise (Io_prim_error "string_length")
  
let printstring = function
  | [Cstring s; c] -> print_endline s; (c, [])
  | _ -> raise (Io_prim_error "print_string")

let printstring_ = function
  | [Cstring s; c] -> print_string s; (c, [])
  | _ -> raise (Io_prim_error "print_string_")
  
let map =
  [("terminate", terminate);
   ("int_add",addint);
   ("int_sub",subint);
   ("int_mul",multint);
   ("int_div",divint);
   ("eq",eqcont);
   ("int_lt",ltint);
   ("int_gt",gtint);
   ("int_print_endline",printint);
   ("int_print",printint_);
   ("string_concat",concstring);
   ("string_print_endline",printstring);
   ("string_print",printstring_);
   ("int_to_string",stringfromint);
   ("string_sub",substring);
   ("string_length",string_length);
   ("string_to_int",intfromstring);]
   