#  Tuple module (tuple.io)
#  Amalthea standard Io library
#  (c)2002 Martin Sandin (msandin@hotmail.com)
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import prelude.
export
  Tuple
  fst
  snd
  both
  TupleIterator
  TuplePrinter
  string_of_tuple
  arg_as_tuple
  apply_tupled
  apply_separate
  curry_func
  Triple.

# Interface
#  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #

declare Tuple: -> first second return;
  return -> client;
  client first second.
  
declare fst: -> tuple return;
  tuple -> first _;
  return first.

declare snd: -> tuple return;
  tuple -> _ second;
  return second.
  
declare both: -> tuple yield return;
  tuple -> first second;
  yield first;
  yield second;
  return.
  
declare TupleIterator: -> tuple return;
  return -> end yield;
  tuple -> first second;
  yield first -> end yield;
  yield second -> end yield;
  end.

declare TuplePrinter: -> string_of_fst string_of_snd return;
  return -> tuple return;
  string_of_tuple tuple string_of_fst string_of_snd -> str;
  print_string str;
  return.
  
declare string_of_tuple: -> tuple string_of_fst string_of_snd return;
  tuple -> fst snd;
  string_of_fst fst -> str_fst;
  ^ "(" str_fst -> str;
  ^ str "," -> str;
  string_of_snd snd -> str_snd;
  ^ str str_snd -> str;
  ^ str ")" -> str;
  return str.
  
declare arg_as_tuple: -> f return;
  return -> tuple return;
  tuple -> fst snd;
  f fst snd;
  return.
  
declare curry_func: -> f return;
  return -> arg1 return;
  return -> arg2 return;
  f arg1 arg2;
  return.

declare apply_separate: -> f tuple return;
  tuple -> a b;
  f a b;
  return.
  
declare apply_tupled: -> f a b return;
  Tuple a b -> tuple;
  f tuple;
  return.
  
# Triples
declare Triple: -> first second third return;
  return -> client;
  client first second third.
  
  
# Private
#  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #

# Module demo 
#  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #

print_string "Amalthea standard Io library: Tuple module";

print_string "Construct a tuple (1,2):";
Tuple 1 2 -> pair;

print_string "Test fst,snd:";
fst pair -> x;
print_int x;
snd pair -> x;
print_int x;

print_string "Test both:";
both pair print_int;

print_string "Tuple iterator:";
TupleIterator pair -> iter;
for_each iter print_int;

print_string "arg_as_tuple + applied to Tuple 1 2:";
arg_as_tuple + -> +pair;
Tuple 1 2 -> pair;
+pair pair -> r;
print_int r;

print_string "Curry_func applied to - (3-7,3-1):";
curry_func - -> _-x;
_-x 3 -> x-_;
x-_ 7 -> r;
print_int r;
x-_ 1 -> r;
print_int r;

terminate